import axios, { AxiosError } from "axios";
import { message } from "ant-design-vue";
import { loadProgressBar } from "axios-progress-bar";
import { signToken } from "./signToken";
import { filterConfigEmpty } from "./configFilter";
import router from "@/routes";

const request = axios.create({
  baseURL: import.meta.env.VITE_APP_API_HOST,
  withCredentials: true,
  timeout: 20000,
  method: "post",
  transformRequest: [],
});

request.interceptors.request.use(
  (config) => {
    if (!config.url) {
      throw new AxiosError(
        "request must should a url parameter",
        "REQUEST_PARAMETER_ERROR",
        config
      );
    }
    config.headers = config.headers || {};
    if (!config.noAuth) {
      const { token, timestamp } = signToken(config.url, config.method);
      config.headers["Authorization"] = token;
      config.headers["time"] = timestamp;
    }
    config.headers["Content-Type"] =
      config.headers["Content-Type"] || "application/x-www-form-urlencoded";
    if (!config.headers["Content-Type"]?.toString().includes("multipart")) {
      config.transformRequest = (d) => new URLSearchParams(d);
    }
    filterConfigEmpty(config);
    return config;
  },
  (error: AxiosError) => {
    if (!error.config.noToast) {
      message.error(error.message || error.name || error.code);
    }
    return Promise.reject(error);
  }
);

request.interceptors.response.use(
  (response) => {
    if (response.config.download) {
      return response;
    }
    if (response.data.hasError) {
      const errMsg = response.data?.errMsg?.join("\n");
      message.error(errMsg);
      return Promise.reject(response.data);
    }
    return response.data;
  },
  (error: AxiosError<any>) => {
    if (error.isAxiosError && !error.config.noToast) {
      const msg =
        error?.response?.data?.message ||
        error.message ||
        error.name ||
        error.code;
      message.error(msg);
    } else if (!error.isAxiosError) {
      error.message && message.error(error.message);
    }
    if (
      error.response &&
      ([401, 403].includes(error.response.status) ||
        error.response.data?.code?.toString()?.startsWith("401"))
    ) {
      router.push({ name: "login" });
    }
    return Promise.reject(error);
  }
);

loadProgressBar(null, request);

export default request;
